@@ -31,10 +31,17 @@ my_attrs(void)
 
 TEST_MAIN()
 {
+	/* adjust these parameters for memory leak testing */
+	/* TODO a way for this test to detect memory leak here. */
+	const int n_iterations = 20;	/* E.g. 200000 */
+	const int freq_parameterized = 2;	/* set 1 to parameterize all, INT_MAX for none */
+
 	SQLINTEGER len, out;
-	int i;
+	int i, j;
 	SQLHSTMT stmt1, stmt2;
 	SQLHSTMT *pcur_stmt = NULL;
+	SQLINTEGER bind1;
+	char bind2[20] = "parameters";
 
 	odbc_use_version3 = true;
 	odbc_set_conn_attr = my_attrs;
@@ -77,18 +84,47 @@ TEST_MAIN()
 	AutoCommit(SQL_AUTOCOMMIT_OFF);
 
 	/* try to do a select which return a lot of data (to test server didn't cache everything) */
-	odbc_command("select a.n, b.n, a.v from #mars1 a, #mars1 b order by a.n, b.n");
+	odbc_command("select a.n, b.n, c.n, a.v from #mars1 a, #mars1 b, #mars1 c order by a.n, b.n, c.n");
 	CHKFetch("S");
 
-	/* try to do other commands */
 	CHKAllocStmt(&stmt2, "S");
-	SET_STMT(stmt2);
-	odbc_command("insert into #mars2 values(1, 'foo')");
-	SET_STMT(stmt1);
 
-	CHKFetch("S");
+	/* Use a parameterized insert. This causes DONEINPROC to be returned by SQL Server,
+	 * leading to result_type==TDS_CMD_DONE when it's complete. Without the parameter,
+	 * result_type == TDS_DONE_RESULT.
+	 * And in odbc_SQLExecute(), it calls odbc_unlock_statement() for TDS_CMD_DONE, but
+	 * not for TDS_DONE_RESULT. (We don't know why...)
+	 * This means that the stmt->tds TDSSOCKET struct is completely freed after every
+	 * iteration of the insert if and only if it was a parameterized insert. So we need
+	 * to test both parameterized and non-parameterized inserts.
+	 */
+	SQLBindParameter(stmt2, 1, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER, 0, 0, &bind1, 0, NULL);
+	SQLBindParameter(stmt2, 2, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 0, 0, &bind2, 20, NULL);
+
+	for (i = 1; i <= n_iterations; ++i) {
+		SET_STMT(stmt2);
+
+		/* Test option - force reallocation of socket
+		 * odbc_reset_statement();
+		 */
+		if (i % freq_parameterized == 0) {
+			bind1 = i;
+			odbc_command("@insert into #mars2 values(?, ?)");
+		} else {
+			odbc_command("@insert into #mars2 values(1, 'foo')");
+		}
+
+		/* Perform several fetches for each insert, so we also test continuing to draw
+		 * further packets of the fetch
+		 */
+		SET_STMT(stmt1);
+		for (j = 0; j < 10; ++j)
+			CHKFetch("S");
+	}
+	printf("Performed %d inserts while fetching.\n", i - 1);
 
 	/* reset statements */
+	SET_STMT(stmt1);
 	odbc_reset_statement();
 	SET_STMT(stmt2);
 	odbc_reset_statement();
@@ -114,8 +150,6 @@ TEST_MAIN()
 
 	EndTransaction(SQL_COMMIT);
 
-	/* TODO test receiving large data should not take much memory */
-
 	odbc_disconnect();
 	return 0;
 }
